define(["require", "exports"], function (require, exports) {
    "use strict";
    (function (PanelType) {
        /**
         * Renders the panel in 'small' mode, anchored to the far side (right in LTR mode), and has a fluid width.
         * Only used on Small screen breakpoints.
         * Small: 320-479px width (full screen), 16px Left/Right padding
         * Medium: <unused>
         * Large: <unused>
         * XLarge: <unused>
         * XXLarge: <unused>
         */
        PanelType[PanelType["smallFluid"] = 0] = "smallFluid";
        /**
         * Renders the panel in 'small' mode, anchored to the far side (right in LTR mode), and has a fixed width.
         * Small: 272px width, 16px Left/Right padding
         * Medium: 340px width, 16px Left/Right padding
         * Large: 340px width, 32px Left/Right padding
         * XLarge: 340px width, 32px Left/Right padding
         * XXLarge: 340px width, 40px Left/Right padding
         */
        PanelType[PanelType["smallFixedFar"] = 1] = "smallFixedFar";
        /**
         * Renders the panel in 'small' mode, anchored to the near side (left in LTR mode), and has a fixed width.
         * Small: 272px width, 16px Left/Right padding
         * Medium: 272px width, 16px Left/Right padding
         * Large: 272px width, 32px Left/Right padding
         * XLarge: 272px width, 32px Left/Right padding
         * XXLarge: 272px width, 32px Left/Right padding
         */
        PanelType[PanelType["smallFixedNear"] = 2] = "smallFixedNear";
        /**
         * Renders the panel in 'medium' mode, anchored to the far side (right in LTR mode).
         * Small: <adapts to smallFluid>
         * Medium: <adapts to smallFixedFar>
         * Large: 48px fixed left margin, 32px Left/Right padding
         * XLarge: 644px width, 32px Left/Right padding
         * XXLarge: 643px width, 40px Left/Right padding
         */
        PanelType[PanelType["medium"] = 3] = "medium";
        /**
         * Renders the panel in 'large' mode, anchored to the far side (right in LTR mode), and is fluid at XXX-Large breakpoint.
         * Small: <adapts to smallFluid>
         * Medium:  <adapts to smallFixedFar>
         * Large: <adapts to medium>
         * XLarge: 48px fixed left margin, 32px Left/Right padding
         * XXLarge: 48px fixed left margin, 32px Left/Right padding
         * XXXLarge: 48px fixed left margin, (no redlines for padding, assuming previous breakpoint)
         */
        PanelType[PanelType["large"] = 4] = "large";
        /**
         * Renders the panel in 'large' mode, anchored to the far side (right in LTR mode), and is fixed at XXX-Large breakpoint.
         * Small: <adapts to smallFluid>
         * Medium:  <adapts to smallFixedFar>
         * Large: <adapts to medium>
         * XLarge: 48px fixed left margin, 32px Left/Right padding
         * XXLarge: 48px fixed left margin, 32px Left/Right padding
         * XXXLarge: 940px width, (no redlines for padding, assuming previous breakpoint)
         */
        PanelType[PanelType["largeFixed"] = 5] = "largeFixed";
        /**
         * Renders the panel in 'extra large' mode, anchored to the far side (right in LTR mode).
         * Small: <adapts to smallFluid>
         * Medium: <adapts to smallFixedFar>
         * Large: <adapts to medium>
         * XLarge: <adapts to large>
         * XXLarge: 176px fixed left margin, 40px Left/Right padding
         * XXXLarge: 176px fixed left margin, 40px Left/Right padding
         */
        PanelType[PanelType["extraLarge"] = 6] = "extraLarge";
    })(exports.PanelType || (exports.PanelType = {}));
    var PanelType = exports.PanelType;
});
